'From Squeak2.7 of 5 January 2000 [latest update: #1782] on 11 November 2000 at 7:58:40 pm'!
"Change Set:		emacs
Date:			11 November 2000
Author:			Chris Reuter


                                     MIT License
                                           
   Copyright (c) 2000 Chris Reuter
   
   Permission is hereby granted, free of charge, to any person obtaining a copy of
   this software and associated documentation files (the 'Software'), to deal in
   the Software without restriction, including without limitation the rights to
   use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
   the Software, and to permit persons to whom the Software is furnished to do so,
   subject to the following conditions:
   
   The above copyright notice and this permission notice shall be included in all
   copies or substantial portions of the Software.
   
   THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
   FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
   COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

In addition to the above, permission is also granted to merge this software with Squeak (i.e. 'file in')
and release the result under the Squeak license.

"
!


!ParagraphEditor methodsFor: 'emacs keys' stamp: 'cr 8/23/1998 02:28'!
cursorBeginningOfLine: characterStream 
	"Private - Move cursor from position in current line to beginning of
	current line. If cursor already at beginning of line, do nothing."
	| string left |

	sensor keyboard.

	string _ paragraph text string.
	left _ startBlock stringIndex.
	[left > 1 and: [(string at: (left - 1)) ~= Character cr]] whileTrue:
		[left _ left - 1].

	startBlock stringIndex == left
		ifFalse: [self selectAt: left].

	^true! !

!ParagraphEditor methodsFor: 'emacs keys' stamp: 'cr 8/23/1998 02:27'!
cursorEndOfLine: characterStream 
	"Private - Move cursor end of current line. If cursor already at end of
	 line, do nothing."
	| string right stringSize |

	sensor keyboard.

	string _ paragraph text string.
	stringSize _ string size.
	right _ stopBlock stringIndex.
	[right <= stringSize and: [(string at: right) ~= Character cr]]
		whileTrue: [right _ right + 1].

	stopBlock stringIndex == right
		ifFalse: [self selectAt: right].

	^true! !

!ParagraphEditor methodsFor: 'emacs keys' stamp: 'cr 8/23/1998 03:44'!
cutToEndOfLine: characterStream 
	"Cut from cursor position to end of line a la emacs CTRL-y. Flushes typeahead."

	| string stringSize right |
	sensor keyboard.		"flush character"

	string _ paragraph text string.
	stringSize _ string size.
	right _ stopBlock stringIndex.
	[right <= stringSize and: [(string at: right) ~= Character cr]]
		whileTrue: [right _ right + 1].

	"Only delete the trailing newline if we're at the end of the line."

	"Unfortunately, this doesn't behave as expected--merging consecutive CTRL-y's--so
	 I've removed it, as there's less chance of data loss."
[
	(right > startBlock stringIndex and: [(string at: right) = Character cr])
		ifTrue: [
			right _ right - 1.
		].
].
	self selectFrom: startBlock stringIndex to: right.
	self cut.
	^true! !

!ParagraphEditor methodsFor: 'typing support' stamp: 'cr 8/23/1998 02:10'!
dispatchOnCharacter: char with: typeAheadStream
	"Carry out the action associated with this character, if any.
	Type-ahead is passed so some routines can flush or use it."

	char = Character enter
		ifTrue: [^ self dispatchOnEnterWith: typeAheadStream].

	"enter, backspace, and escape keys (ascii 3, 8, and 27) are command keys"
	(sensor commandKeyPressed 
		or: [(self class specialShiftCmdKeys includes: char asciiValue) 
			and: [sensor controlKeyPressed not]]) ifTrue:
		[^ sensor leftShiftDown
			ifTrue:
				[self perform: (ShiftCmdActions at: char asciiValue + 1) with: typeAheadStream]
			ifFalse:
				[self perform: (CmdActions at: char asciiValue + 1) with: typeAheadStream]].

	"the control key can be used to invoke shift-cmd shortcuts"
	sensor controlKeyPressed ifTrue:
		[^ self perform: (ShiftCmdActions at: char asciiValue + 1) with: typeAheadStream].
	^ self perform: #normalCharacter: with: typeAheadStream! !


!ParagraphEditor class reorganize!
('class initialization' initialize initializeTextEditorMenus shiftedYellowButtonMenu yellowButtonMenu yellowButtonMessages)
('instance creation' new newParagraph:)
('keyboard shortcut tables' initializeCmdKeyShortcuts initializeShiftCmdKeyShortcuts specialShiftCmdKeys)
('emacs key bindings' defaultBindings emacsBindings emacsControlKeyShortcuts)
('clipboard access' clipboardContents clipboardText clipboardTextPut:)
!


!ParagraphEditor class methodsFor: 'keyboard shortcut tables' stamp: 'cr 8/23/1998 04:10'!
initializeCmdKeyShortcuts
	"Initialize the (unshifted) command-key shortcut table."
	"NOTE: if you don't know what your keyboard generates, use Sensor kbdTest"
	"ParagraphEditor initialize"

	| cmdMap cmds |
	cmdMap _ Array new: 256.  "use temp in case of a crash"
	cmdMap atAllPut: #noop:.
	cmdMap at: ( 1 + 1) put: #cursorHome:.			"home key"
	cmdMap at: ( 4 + 1) put: #cursorEnd:.			"end key"
	cmdMap at: ( 8 + 1) put: #backspace:.			"ctrl-H or delete key"
	cmdMap at: (13 + 1) put: #crWithIndent:.			"cmd-Return"
	cmdMap at: (27 + 1) put: #selectCurrentTypeIn:.	"escape key"
	cmdMap at: (28 + 1) put: #cursorLeft:.			"left arrow key"
	cmdMap at: (29 + 1) put: #cursorRight:.			"right arrow key"
	cmdMap at: (30 + 1) put: #cursorUp:.				"up arrow key"
	cmdMap at: (31 + 1) put: #cursorDown:.			"down arrow key"

	'0123456789-=' do: [ :char | cmdMap at: (char asciiValue + 1) put: #changeEmphasis: ].
	'([{''"<'         do: [ :char | cmdMap at: (char asciiValue + 1) put: #enclose: ].
	cmdMap at: ($, asciiValue + 1) put: #shiftEnclose:.

	cmds _ #(
		$a	selectAll:
		$b	browseIt:
		$c	copySelection:
		$d	doIt:
		$e	exchange:
		$f	find:
		$g	findAgain:
		$h	setSearchString:
		$i	inspectIt:
		$j	doAgainOnce:
		$k  offerFontMenu:
		$l	cancel:
		$m	implementorsOfIt:
		$n	sendersOfIt:
		$o	spawnIt:
		$p	printIt:
		$q	querySymbol:
		$r	recognizer:
		$s	save:
		$t	tempCommand:
		$u	align:
		$v	paste:
		$w	backWord:
		$x	cut:
		$y	swapChars:
		$z	undo:
	).
	1 to: cmds size by: 2 do: [ :i |
		cmdMap at: ((cmds at: i) asciiValue + 1) put: (cmds at: i + 1).
	].

	"Make the changes take effect immediately on all instances."
	CmdActions notNil
		ifTrue: [
			cmdMap withIndexDo: [ :value :index | CmdActions at: index put: value]
		] ifFalse: [	
			CmdActions _ cmdMap.
		].! !

!ParagraphEditor class methodsFor: 'keyboard shortcut tables' stamp: 'cr 8/23/1998 04:12'!
initializeShiftCmdKeyShortcuts
	"Initialize the shift-command-key (or control-key) shortcut table."
	"NOTE: if you don't know what your keyboard generates, use Sensor kbdTest"

	| cmdMap cmds |
	"shift-command and control shortcuts"
	cmdMap _ Array new: 256.  "use temp in case of a crash"
	cmdMap atAllPut: #noop:.
	cmdMap at: ( 1 + 1) put: #cursorHome:.			"home key"
	cmdMap at: ( 4 + 1) put: #cursorEnd:.			"end key"
	cmdMap at: ( 8 + 1) put: #forwardDelete:.			"ctrl-H or delete key"
	cmdMap at: (13 + 1) put: #crWithIndent:.			"ctrl-Return"
	cmdMap at: (27 + 1) put: #selectCurrentTypeIn:.	"escape key"
	cmdMap at: (28 + 1) put: #cursorLeft:.			"left arrow key"
	cmdMap at: (29 + 1) put: #cursorRight:.			"right arrow key"
	cmdMap at: (30 + 1) put: #cursorUp:.				"up arrow key"
	cmdMap at: (31 + 1) put: #cursorDown:.			"down arrow key"
	cmdMap at: (45 + 1) put: #changeEmphasis:.		"cmd-sh-minus"
	cmdMap at: (61 + 1) put: #changeEmphasis:.		"cmd-sh-plus"
	cmdMap at: (127 + 1) put: #forwardDelete:.		"del key"

	"Note: Command key overrides shift key, so, for example, cmd-shift-9 produces $9 not $("
	'9[,''' do: [ :char | cmdMap at: (char asciiValue + 1) put: #shiftEnclose: ].	"({< and double-quote"
	"Note: Must use cmd-9 or ctrl-9 to get '()' since cmd-shift-9 is a Mac FKey command."
	cmdMap at: (27 + 1) put: #shiftEnclose:.	"ctrl-["

	cmds _ #(
		$a	argAdvance:
		$b	browseItHere:
		$c	compareToClipboard:
		$d	duplicate:
		$e	methodStringsContainingIt:
		$f	displayIfFalse:
		$j	doAgainMany:
		$k	changeStyle:
		$n	referencesToIt:
		$r	indent:
		$l	outdent:
		$s	search:
		$t	displayIfTrue:
		$u	changeLfToCr:
		$v	pasteInitials:
		$w	methodNamesContainingIt:
		$x	makeLowercase:
		$y	makeUppercase:
		$z	makeCapitalized:
	).
	1 to: cmds size by: 2 do: [ :i |
		cmdMap at: ((cmds at: i) asciiValue + 1)			put: (cmds at: i + 1).
		cmdMap at: (((cmds at: i) asciiValue - 96) + 1)	put: (cmds at: i + 1).
	].

	"Make the changes take effect immediately on all instances."
	ShiftCmdActions notNil
		ifTrue: [
			cmdMap withIndexDo: [ :value :index | ShiftCmdActions at: index put: value]
		] ifFalse: [	
			ShiftCmdActions _ cmdMap.
		].
! !

!ParagraphEditor class methodsFor: 'emacs key bindings' stamp: 'cr 8/23/1998 04:26'!
defaultBindings
	"Use default Squeak bindings."

	self initializeCmdKeyShortcuts.
	self initializeShiftCmdKeyShortcuts.
! !

!ParagraphEditor class methodsFor: 'emacs key bindings' stamp: 'cr 8/23/1998 04:06'!
emacsBindings
	"Use emacs bindings.  Yay!!"
	self emacsControlKeyShortcuts
! !

!ParagraphEditor class methodsFor: 'emacs key bindings' stamp: 'cr 8/23/1998 04:19'!
emacsControlKeyShortcuts
	"Initialize the shift-command-key (or control-key) shortcut table."
	"NOTE: if you don't know what your keyboard generates, use Sensor kbdTest"

	| cmdMap cmds |
	"shift-command and control shortcuts"
	cmdMap _ Array new: 256.  "use temp in case of a crash"
	cmdMap atAllPut: #noop:.
	cmdMap at: ( 1 + 1) put: #cursorHome:.			"home key"
	cmdMap at: ( 4 + 1) put: #cursorEnd:.			"end key"
	cmdMap at: ( 8 + 1) put: #forwardDelete:.			"ctrl-H or delete key"
	cmdMap at: (13 + 1) put: #crWithIndent:.			"ctrl-Return"
	cmdMap at: (27 + 1) put: #selectCurrentTypeIn:.	"escape key"
	cmdMap at: (28 + 1) put: #cursorLeft:.			"left arrow key"
	cmdMap at: (29 + 1) put: #cursorRight:.			"right arrow key"
	cmdMap at: (30 + 1) put: #cursorUp:.				"up arrow key"
	cmdMap at: (31 + 1) put: #cursorDown:.			"down arrow key"
	cmdMap at: (45 + 1) put: #changeEmphasis:.		"cmd-sh-minus"
	cmdMap at: (61 + 1) put: #changeEmphasis:.		"cmd-sh-plus"
	cmdMap at: (127 + 1) put: #forwardDelete:.		"del key"

	"Note: Command key overrides shift key, so, for example, cmd-shift-9 produces $9 not $("
	'9[,''' do: [ :char | cmdMap at: (char asciiValue + 1) put: #shiftEnclose: ].	"({< and double-quote"
	"Note: Must use cmd-9 or ctrl-9 to get '()' since cmd-shift-9 is a Mac FKey command."
	cmdMap at: (27 + 1) put: #shiftEnclose:.	"ctrl-["

	cmds _ #(
		$a	cursorBeginningOfLine:
		$b	cursorLeft:
		$f	cursorRight:
		$n	cursorDown:
		$p	cursorUp:
		$d	forwardDelete:
		$e	cursorEndOfLine:
		$s	find:
		$k	cutToEndOfLine:
		$y	paste:
		$t	swapChars:
		$w	cut:
	).
	1 to: cmds size by: 2 do: [ :i |
		cmdMap at: ((cmds at: i) asciiValue + 1)			put: (cmds at: i + 1).
		cmdMap at: (((cmds at: i) asciiValue - 96) + 1)	put: (cmds at: i + 1).
	].

	"Cleverness to make the changes take effect immediately."
	ShiftCmdActions class = Array
		ifTrue: [
			cmdMap withIndexDo: [ :value :index | ShiftCmdActions at: index put: value]
		] ifFalse: [	
			ShiftCmdActions _ cmdMap.
		].! !


